module OfflineDeployer
  class Maker
    def initialize(from_tag = nil, to_tag = nil, destination = nil)
      @from_tag = from_tag.presence || default_from_tag
      @to_tag = to_tag.presence || default_to_tag
      @destination = destination || default_destination
      @long_name = from_tag.present?
    end

    attr_reader :from_tag, :to_tag, :destination, :long_name

    def call
      pull_and_bundle unless Rails.env.development?
      write_update_info_and_deleted_files
      add_files_to_archive
      handle_gems
      move_archive_to_destination
      print_release_info
      return_file_path_and_name
    end

    private

    def app_root
      OfflineDeployer.settings.app_root
    end

    def pull_and_bundle
      Dir.chdir(app_root) do
        `git pull`
        Bundler.with_clean_env do
          `bundle install`
        end
      end
    end

    def tags
      Tagger.new.tag_list
    end

    def default_from_tag
      tags[1]
    end

    def default_to_tag
      tags[0]
    end

    def default_destination
      "#{app_root}/tmp/releases/versions/"
    end

    def write_update_info_and_deleted_files
      File.open('.update', 'w+') do |f|
        f.write(
          {
            FROM_VERSION: from_tag,
            TO_VERSION: to_tag,
            DELETED_FILES: get_file_names(deleted_files)
          }.to_yaml
        )
      end

      `mv .update #{app_root}` unless Rails.env.development?
    end

    def changed_files
      @changed_files ||= `cd #{app_root} && git diff --name-status #{from_tag} #{to_tag}`.split("\n")
    end

    def deleted_files
      @deleted_files ||= changed_files.select { |file| file.starts_with?('D') }
    end

    def archive_files
      @archive_files ||= get_file_names(changed_files - deleted_files)
    end

    def get_file_names(files_array)
      files_array.map { |file| file.split("\t")[1] }
    end

    def add_files_to_archive
      `cd #{app_root} && git archive -o #{archive_file_path} #{to_tag} #{archive_files.join(' ')}`
      `cd #{app_root} && zip #{archive_file_path} .update`
    end

    def archive_file_name
      long_name ? "#{archive_name_prefix}-#{from_tag}-#{to_tag}.zip" : "#{archive_name_prefix}-#{to_tag}.zip"
    end

    def archive_file_path
      "#{app_root}/#{archive_file_name}"
    end

    def archive_name_prefix
      OfflineDeployer.settings.archive_name_prefix
    end

    def handle_gems
      OfflineDeployer::GemsHandler.new(from_tag, to_tag, app_root, archive_file_path).call
    end

    def move_archive_to_destination
      FileUtils.mkdir_p destination
      `mv #{archive_file_path} #{destination}`
    end

    def print_release_info
      puts 'Added files:'
      puts archive_files.join("\n")
      puts "FROM_VERSION: #{from_tag}, TO_VERSION: #{to_tag}"
      puts "Created #{archive_file_name} in #{destination}"
    end

    def return_file_path_and_name
      ["#{destination}/#{archive_file_name}", archive_file_name]
    end
  end
end
